
#ifndef AFGCOCKPIT_H
#define AFGCOCKPIT_H

#include <Arduino.h> //needed for Serial.println

// Debug mode
// Uncomment to enable debug mode. If so, additional messages will be printed 
// on serial, which can be useful together with the serial monitor. FlightGear will 
// however behave strangely with this enabled, so don't forget to turn off!
//#define ARDEBUG

#define SERIALBAUDRATE       38400

// The delay in microseconds for use with the serial read functions. If we are 
// supposed to read data but a variable separator or newline has not been read
// we will wait this amount of time and re-read:
#define SERIALREADWAIT 100

// Same separator as defined in protocol: <var_separator>,</var_separator>
#define PROTOCOL_VARSEP  ','

//       Heartbeat
//       ---------
// Enable heartbeat to let Arduino show heartbeat by flashing an LED.
// See heartbeat.cpp for more information.
#define HEARTBEAT
#ifdef HEARTBEAT
const int heartbeat_ledpin = 13;
#endif

//       Sync LED
//       --------
// Flash an LED if Arduino and FG are in sync. If un-synched (just after Arduino has
// sent a message) the LED will be off.
#define SYNCLED

// Configuration:
#ifdef SYNCLED
const int syncled_ledpin = 12;
#endif

// Sensor types.
// Analog pots can be of two types (have two modes); absolute or relative.
// Absolute will send the absolute read value and update the parameter tree.
// Relative will send the changed value, so 0 if no change since last transmission.
// It relies on Nasal code to update correct value in parameter tree. This is how
// encoders are used.
// Muxers have also implemented MIMIC_PUSH_BUTTON which means that a momentary
// button can be pressed once and it will hold the state until pressed again.
enum SensorType {
  ABSOLUTE_VALUE,
  RELATIVE_VALUE,
  MIMIC_PUSH_BUTTON,
};


// Analog potentiometers.
// Define the number of analog pots directly connected to an analog input.
#define NANALOGPOTS 1

#if NANALOGPOTS > 0
// Define which pin and also name it here (so it is easier to modify configuration from the default).
const int analogpots_pins[] = { 
    0,
//    1,
//    2,
    };
enum AnalogPotOrder {
    ELEVATOR_TRIM,
//    ELEVATOR,
//    THROTTLE,
    };
#endif


// Digital pushbuttons.
// Define the number of push buttons directly connected to a digital input.
#define NPUSHBUTTONS 1
#if NPUSHBUTTONS > 0
const int pushbutton_pins[] = { 7 };
enum PushButtonOrder { APON };
#endif

// Encoders.
// Define the number of encoders used.
#define NENCODERS 1
#if NENCODERS > 0
enum EncoderOrder { ENC_RUDDER };
// The pins must be defined in pairs:
const int encoder_pins[] = { 2,3 };
#endif

// Set the total number of elements (IC chips) in the shift register chain:
#define NSHIFTREGISTERS 8
#if NSHIFTREGISTERS > 0

//Pin connected to ST_CP (12) of 74HC595
#define SHIFTREGISTER_LATCHPIN  4
//Pin connected to SH_CP (11) of 74HC595
#define SHIFTREGISTER_CLOCKPIN  5
////Pin connected to DS (14) of 74HC595
#define SHIFTREGISTER_DATAPIN   6

// Name the different sections in the chain and do the setup of these
// in the setup() function.
enum ShiftRegisterOrder {
      ADFLEFT,
      ADFRIGHT,
      __SHIFTREGISTERLAST__
};


#endif

// Muxers.
// Define how many mux chips are used.
#define NMUXERCONTROLLERS 2
#if NMUXERCONTROLLERS > 0

#define MUX_PIN_S0 10
#define MUX_PIN_S1 9
#define MUX_PIN_S2 8

// Define total number of channels connected to all muxer chips.
// The number should match the numbers in 'MuxOrder' and 'muxer_address'.
#define NMUXERCHANNELS 8
const int muxer_analogin_pins[] = { A5, A4 };
enum MuxOrder {
      ADFADF,
      ADFBFO,
      ADFFRQ,
      ADFFLT,
      ADFSET,
      AILERON,
      ELEVATOR,
      THROTTLE,
      //APUP,
      //APDOWN
    };
// Setup where these are located, they are in pairs: mux,channel, mux,channel, ...
const int muxer_address[] = {
      0,3,
      0,0,
      0,1,
      0,5,
      0,7,
      1,0,
      1,2,
      1,1,
    };


#endif


void PrintDouble( double val, byte precision);
double mapf(float x, float in_min, float in_max, float out_min, float out_max);
int OutsideRange(float val, float old, float range);
int analog2discrete(int value, int minv, int maxv, int steps);

#endif
