
#ifndef ANALOGPOTS_H
#define ANALOGPOTS_H

#include <Arduino.h> //needed for Serial.println

typedef struct  {
  int pin = 0;        // pin to read values from
  float value = 0;
  float raw_value = 0;
  int minin = 0;
  int maxin = 1023;
  float minout = -1;
  float maxout = 1;
  int accuracy = 3;
  int measure_averages = 1;
  int measure_delay = 1000;
  int precision = 3;
  int mode = ABSOLUTE_VALUE;
  // Analog pots can be of two types (have two modes); absolute or relative.
  // Absolute will send the absolute read value and update the parameter tree.
  // Relative will send the changed value, so 0 if no change since last transmission.
  // It relies on Nasal code to update correct value in parameter tree. This is how
  // encoders are used.
  // Time when value was read last time
  unsigned long timer = 0; // unit: ms
  // Only read if this amount of time (in ms) has passed (default value
  // for all sensors read from the muxers, adjust individuals in setup fn).
  // A small value reduces loop frequency, a high value increases
  // the risk of not catching a presses on buttons
  unsigned int delay = 50;
} AnalogPot;

void setupAnalogPots();
void readAnalogPots();
void readAnalogPot(AnalogPot *pot);
void printAnalogPots();
void clearAnalogPots();

extern int sendmsg;
extern AnalogPot analogpots[NANALOGPOTS];


#endif
