
// encoders.cpp

/*******Interrupt-based Rotary Encoder Sketch*******
by Simon Merrett, based on insight from Oleg Mazurov, Nick Gammon, rt, Steve Spence
Source: https://www.instructables.com/id/Improved-Arduino-Rotary-Encoder-Reading/
*/

#include "afgcockpit.h"
#include "encoders.h"



#if NENCODERS > 0


void setupEncoders() {
  for (int i=0; i<NENCODERS; i++) {
    encoder[i].pinA = encoder_pins[i*2];
    encoder[i].pinB = encoder_pins[i*2+1];  // Encoder pins are in pairs.
    pinMode(encoder[i].pinA, INPUT_PULLUP); // set pinA as an input, pulled HIGH to the logic voltage (5V or 3.3V for most cases)
    pinMode(encoder[i].pinB, INPUT_PULLUP); // set pinB as an input, pulled HIGH to the logic voltage (5V or 3.3V for most cases)
    #ifdef ARDEBUG
    Serial.println("Encoder:");
    Serial.println(encoder[i].pinA);
    Serial.println(encoder[i].pinB);
    #endif
  }
  attachInterrupt(digitalPinToInterrupt(encoder[0].pinA),Enc0A,RISING); // set an interrupt on PinA, looking for a rising edge signal and executing the "PinA" Interrupt Service Routine (below)
  attachInterrupt(digitalPinToInterrupt(encoder[0].pinB),Enc0B,RISING); // set an interrupt on PinB, looking for a rising edge signal and executing the "PinB" Interrupt Service Routine (below) 
}

// Add more functions as necessary for each encoder:
void Enc0A(){ readEncoderPinA(&encoder[0]); }
void Enc0B(){ readEncoderPinB(&encoder[0]); }


void calculateEncoders(){
  for (int i=0; i<NENCODERS; i++) {
    if(encoder[i].oldEncPos != encoder[i].encoderPos) {
      if (encoder[i].encoderPos == 0 && encoder[i].oldEncPos == ENCODERMAXVALUE)
        encoder[i].change++;
      else if (encoder[i].encoderPos == ENCODERMAXVALUE && encoder[i].oldEncPos == 0)
        encoder[i].change--;
      else
        encoder[i].change = encoder[i].change + encoder[i].encoderPos - encoder[i].oldEncPos;
//      Serial.print(encoder[i].oldEncPos);
//      Serial.print("\n");
//      Serial.print(encoder[i].encoderPos);
//      Serial.print("\n");
//      Serial.print(encoder[i].change);
//      Serial.print("\n");
      encoder[i].oldEncPos = encoder[i].encoderPos;
      sendmsg = true;
    }
  }
}

void printEncoders(const char *separator) {
  for (int i=0; i<NENCODERS; i++) {
    Serial.print(encoder[i].change);
    Serial.print(separator);
  }
}

void clearEncoders() {
  for (int i=0; i<NENCODERS; i++)
    encoder[i].change = 0;
}



// Generic functions to read encoder values:

void readEncoderPinA(Encoder *enc) {
  cli(); //stop interrupts happening before we read pin values
  enc->reading = PIND & 0xC; // read all eight pin values then strip away all but pinA and pinB's values
  if(enc->reading == B00001100 && enc->aFlag) { //check that we have both pins at detent (HIGH) and that we are expecting detent on this pin's rising edge
    enc->encoderPos --; //decrement the encoder's position count
    enc->bFlag = 0; //reset flags for the next turn
    enc->aFlag = 0; //reset flags for the next turn
  }
  else if (enc->reading == B00000100)
    enc->bFlag = 1; //signal that we're expecting pinB to signal the transition to detent from free rotation 
  sei(); //restart interrupts
}

void readEncoderPinB(Encoder *enc) {
  cli(); //stop interrupts happening before we read pin values
  enc->reading = PIND & 0xC; //read all eight pin values then strip away all but pinA and pinB's values
  if (enc->reading == B00001100 && enc->bFlag) { //check that we have both pins at detent (HIGH) and that we are expecting detent on this pin's rising edge
    enc->encoderPos ++; //increment the encoder's position count
    enc->bFlag = 0; //reset flags for the next turn
    enc->aFlag = 0; //reset flags for the next turn
  }
  else if (enc->reading == B00001000)
    enc->aFlag = 1; //signal that we're expecting pinA to signal the transition to detent from free rotation
  sei(); //restart interrupts
}



#endif
