// lcd.cpp

#include "afgcockpit.h"
#include "lcd.h"

char char2LED7(char c) {
  // Returns the bits to create letters of a 7-segment LED display. It is assumed 
  // that the bits are in same order as connected in hardware. 
  // To turn off a segment it should be 1, hence the byte is XOR:ed at the end.
  
  char res = 0;

  // We assume that the 7 segment display has the 8 LED's position like this:
  //
  //      ---      a
  //     /  /    f  b
  //    ----      g
  //   /  /     e  c
  //   --- .     d  h
  //
  // The characters are defined in a byte (variable 'res') with these bits and order:
  //   hgfedcba
  // So to write number '4' we need to light up f,g,b,c LED's.
  // The shift register is connected in this order:
  // pin 0 -> segment a
  // pin 1 -> segment b
  // ....
  // pin 7 -> segment h


  //  g f C a b
  // 10 9 8 7 6
  //
  //  1 2 3 4 5
  //  e d C c h   C=common
  
  switch(c) {
    case 0:
    case '0':  res = B00111111;  break;
    case 1:    
    case '1':  res = B00000110;  break;
    case 2:    
    case '2':  res = B01011011;  break;
    case 3:    
    case '3':  res = B01001111;  break;
    case 4:    
    case '4':  res = B01100110;  break;
    case 5:    
    case '5':  res = B01101101;  break;
    case 6:    
    case '6':  res = B01111101;  break;
    case 7:    
    case '7':  res = B00000111;  break;
    case 8:    
    case '8':  res = B01111111;  break;
    case 9:    
    case '9':  res = B01100111;  break;
    
    case 'A':
    case 'a':  res = B01110111;  break;
    case 'B':
    case 'b':  res = B01111100;  break;
    case 'C':  res = B00111001;  break;
    case 'c':  res = B01011000;  break;
    case 'D':  
    case 'd':  res = B01011110;  break;
    case 'E':
    case 'e':  res = B01111001;  break;
    case 'F':
    case 'f':  res = B01110001;  break;
    case 'G':
    case 'g':  res = B00111101;  break;
    case 'H':
    case 'h':  res = B01110100;  break;
    case 'I':
    case 'i':  res = B00110000;  break;
    case 'J':
    case 'j':  res = B00001110;  break;
    case 'K':
    case 'k':  res = B11110110;  break;
    case 'L':
    case 'l':  res = B00111000;  break;
//    case 'M':
//    case 'm':  res = 
    case 'N':
    case 'n':  res = B01010100;  break;
    case 'O':
    case 'o':  res = B01011100;  break;
    case 'P':
    case 'p':  res = B01110011;  break;
    case 'Q':
    case 'q':  res = B10111111;  break;
    case 'R':
    case 'r':  res = B01010000;  break;
    case 'S':  
    case 's':  res = B01101101;  break;
    case 'T':
    case 't':  res = B01111000;  break;
    case 'U':  
    case 'u':  res = B00111110;  break;
    case 'V':
    case 'W':
    case 'w':
    case 'v':  res = B00111100;  break;
    case 'X':
    case 'x':  res = B01110110;  break;
    case 'Y':
    case 'y':  res = B01101110;  break;
    case 'Z':
    case 'z':  res = B01011011;  break;
    
    case '-':  res = B01000000;  break;
    case '_':  res = B00001000;  break;
    case ' ':  res = B00000000;  break;
    case '.':  res = B10000000;  break;
    
    default:   res = B00001000;  break;
  }
  
  res = res ^ B11111111;
  return(res); 
}
