
#ifndef MUX_H
#define MUX_H

#include <Arduino.h> //needed for Serial.println

typedef struct  {
  // Define which mux and which channel to read from:
  int mux_controller = 0;
  int mux_channel = 0;
  // Current value is stored here, it is adjusted so it is
  // within range [minout:maxout]
  float value = 0;
  // Store old value (as read by analogRead() ):
  float raw_value = 0;
  // Define values read from analog in:
  int minin = 0;
  int maxin = 1023;
  // Define which values the value from analog should map (convert) to.
  // Use [0:1] for a push button, [-1:1] for an aileron for example:
  float minout = 0;
  float maxout = 1;
  // If the read analog value is within raw_value+-accuracy it will be
  // discarded. Needed to handle noise.
  float accuracy = 1;
  int measure_averages = 1;
  int measure_delay = 1000;
  // Precision of the value sent to FG. Use 0 for a push button,
  // 3 for a potentiometer value.
  int precision = 0;
  // Set to 1 if this should be treated as a push switch button
  // that will change values between HIGH and LOW for each press.
  // Otherwise will buttons be momentary.
  // Mode of the sensor. Default is ABSOLUTE_VALUE which means it
  // will send the value as-is. For a analog pot it is usually -1 to 1
  // and for a button 0 or 1. Set to MIMIC_PUSH_BUTTON to let a momentary
  // button behave as it can stick until pressed again.
  int mode = ABSOLUTE_VALUE;
  // Time when value was read last time
  unsigned long timer = 0; // unit: ms
  // Only read if this amount of time (in ms) has passed (default value
  // for all sensors read from the muxers, adjust individuals in setup fn).
  // A small value reduces loop frequency, a high value increases
  // the risk of not catching a presses on buttons
  unsigned int delay = 50;
} MuxAnalog;


extern int sendmsg;
#if NMUXERCONTROLLERS > 0
extern MuxAnalog mux[NMUXERCHANNELS];
#endif


void setupMuxers();
void readMuxers();
void readMuxer(MuxAnalog *pot);
void printMuxers();
void clearMuxers();

#endif
