// shiftreg.cpp

#include <stdio.h>
#include "afgcockpit.h"
#include "shiftreg.h"
#include "strings.h"

#if NSHIFTREGISTERS > 0


void setupShiftRegisters() {
  #ifdef ARDEBUG
  Serial.print("setup shift registers (length=");
  Serial.print(NSHIFTREGISTERS);
  Serial.println(")");
  #endif

  // Shift Register:
  pinMode(shift_register.latchPin, OUTPUT);
  pinMode(shift_register.clockPin, OUTPUT);
  pinMode(shift_register.dataPin, OUTPUT);
  // Clear contents:
  resetShiftRegisterChars(0, NSHIFTREGISTERS, 0);

  if (shift_register.calculated_length != NSHIFTREGISTERS) {
    Serial.println("*** WARNING *** Shift register chain length mismatch!");
  }
}

void defineSRdisplay(ShiftRegisterSection *section, int start, int length) {
  if (start < 0)
    start = shift_register.calculated_length;
  section->start = start;
  section->length = length;
  shift_register.calculated_length += length;
}

void writeShiftRegisterInt(ShiftRegisterSection *section, int value) {
  char str[SHIFTREGISTER_SECTION_MAXLENGTH];
  sprintf(str, "%d", value);
  _copyToShiftRegister(section, str);
}

void writeShiftRegisterStr(ShiftRegisterSection *section, char *str) {
  _copyToShiftRegister(section, str);
}


void _readShiftRegister(int start, int length, char *str) {
  if (start < 0 || start+length > NSHIFTREGISTERS)
    return;
  for (int i=0; i<length; i++)
    str[i] = shift_register.data[start+i];
//  str[length]='\0';
}

void _copyToShiftRegister(ShiftRegisterSection *section, char *str) {
// write the null-terminated string 'str' to a section of the shift register data variable.

  // Variable to hold the old value:
  char old[SHIFTREGISTER_SECTION_MAXLENGTH];
  _readShiftRegister(section->start, section->length, old);
  // (Possibly) updated string, align it accordingly:
  char tobe[SHIFTREGISTER_SECTION_MAXLENGTH];
  alignString(str, tobe, section->length, section->alignment, section->paddingchar);

  // No action needed if the string has not changed:
  if (strequal(old, tobe, section->length) == true)
    return;

  for (int i=0; i<section->length; i++) {
    shift_register.data[section->start+i] = tobe[i];
  }

  shift_register.data_changed = true;
  
}


void alignString(char *src, char *to, int totallen, TextAlign align, char paddingchar) {
  int srclen = strlen(src);
  int offset = 0;
  int copybytes = totallen;
  if (srclen < totallen) {
    copybytes = srclen;
    if (align == TEXTALIGN_RIGHT)
      offset = totallen - srclen;
  }


  for (int i=0; i<offset; i++)
    to[i] = char2LED7(paddingchar);
  for (int i=0; i<copybytes; i++)
    to[i+offset] = char2LED7(src[i]);
  for (int i=copybytes+offset; i<totallen; i++)
    to[i] = char2LED7(paddingchar);
//  to[totallen]='\0';
}

void resetShiftRegisterChars(int start, int length, char value) {
  if (start < 0 || start+length > NSHIFTREGISTERS)
    return;

  for (int i=start; i<start+length; i++)
    shift_register.data[i] = value;
}

void shiftoutShiftRegisters() {

  // If nothing changed since last shiftout, no need to do it again:
  if (shift_register.data_changed == false)
    return;

  if (shift_register.timer > millis()+shift_register.update_delay)
    return;
  shift_register.timer = millis();

  digitalWrite(shift_register.latchPin, LOW);
  for (int i=NSHIFTREGISTERS-1; i>=0; i--) {
    shiftOut(shift_register.dataPin, shift_register.clockPin, MSBFIRST, shift_register.data[i]);
  }
  digitalWrite(shift_register.latchPin, HIGH);
  #ifdef ARDEBUG
  printShiftRegistersToSerial();
  #endif

  shift_register.data_changed = false;
}

void printShiftRegistersToSerial() {
  Serial.println("Shift register data:");
  for (int i=0; i<NSHIFTREGISTERS-1; i++) {
    Serial.print(i);
    Serial.print(": ");
    for (int b=7; b>=0; b--) {
      Serial.print(bitRead(shift_register.data[i], b));
    }
    Serial.println("");
  }
}

#endif
