
#ifndef SHIFTREG_H
#define SHIFTREG_H

#include <Arduino.h> //needed for Serial.println
#include "lcd.h"

#if NSHIFTREGISTERS > 0

#define SHIFTREGISTER_SECTION_MAXLENGTH 10

typedef struct {
  //Pin connected to ST_CP (12) of 74HC595
  const int latchPin = SHIFTREGISTER_LATCHPIN;
  //Pin connected to SH_CP (11) of 74HC595
  const int clockPin = SHIFTREGISTER_CLOCKPIN;
  ////Pin connected to DS (14) of 74HC595
  const int dataPin = SHIFTREGISTER_DATAPIN;

  // The data that will be shifted out is stored here:
  char data[NSHIFTREGISTERS];

  // Used to calculate the length when the shift register is
  // initiated:
  int calculated_length = 0;

  bool data_changed = true;
  // Timer when we last shifted out:
  unsigned long timer = 0;
  // (minimum) delay until next update. Set to 0
  // to update the registers each loop if data has changed.
  // This value is checked both before updating the data string
  // and before actually shifting out all data to the chips.
  const int update_delay = 100;


} ShiftRegister;

enum TextAlign {
  TEXTALIGN_RIGHT,
  TEXTALIGN_LEFT,
};

typedef struct {
  int start = 0;
  int length = 0;
  TextAlign alignment = TEXTALIGN_RIGHT;
  char paddingchar = '_';
} ShiftRegisterSection;


extern ShiftRegister shift_register;


void setupShiftRegisters();
void defineSRdisplay(ShiftRegisterSection *section, int start, int length);
void writeShiftRegisterInt(ShiftRegisterSection *section, int value);
void writeShiftRegisterStr(ShiftRegisterSection *section, char *str);
void _readShiftRegister(int start, int length, char *str);
void _copyToShiftRegister(ShiftRegisterSection *section, char *str);
bool strequal(char *s1, char *s2, int length);
void alignString(char *src, char *to, int totallen, TextAlign align, char paddingchar);
void resetShiftRegisterChars(int start, int length, char value);
void shiftoutShiftRegisters();
void printShiftRegistersToSerial();

#endif

#endif
